<?php
Namespace App\Controller\Admin;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Validator\Validation;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Translation\TranslatorInterface;
use Symfony\Component\Filesystem\Filesystem;
use App\Utils\Utils;
use \Homegear\Homegear as Homegear;

class ChangePwdController extends Controller
{
    private $hg;
    private $utils;
    private $translator;

    public function __construct(TranslatorInterface $translator, Utils $utils)
    {
        $this->hg = new Homegear();
        $this->translator = $translator;
        $this->utils = $utils;
    }

    public function indexAction(Request $request)
    {
        $this->_checkPwdFile();
        return $this->render('Admin/changePwd.html.twig',array('mgmt' => $this->utils->checkManagementRunning()));
    }

    public function saveAction(Request $request)
    {
        $this->_checkPwdFile();
        // get the data-path
        if ($this->utils->checkManagementRunning(false)) {
            $hgPath = $this->hg->managementGetConfigurationEntry('main.conf','writeableDataPath');
            if (!$hgPath || $hgPath == '') {
                $hgPath = $this->hg->managementGetConfigurationEntry('main.conf','dataPath');    
            }
        } else {
            // homegear-management is not running or busy right now, so let's do it the classic way
            $hgPath = $this->utils->getDataPath();
        }
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'fields' => array(
                'user' => new Assert\NotBlank,
                'password' => new Assert\NotBlank
            ),
            'allowExtraFields' => true
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            try {
                $this->hg->updateUser($params['user'],$params['password']);
            } catch(\Homegear\HomegearException $e) {
                return new JsonResponse(['result' => 'error', 'message' => $this->translator->trans('changePwd.exception',array(),'admin')]);
            }
            $fs = new Filesystem();
            $fs->remove($hgPath.'/defaultPassword.txt');
            return new JsonResponse(['result' => 'success', 'message' => $this->translator->trans('changePwd.success',array(),'admin')]);
        } else {
            return new JsonResponse(['result' => 'error', 'message' => $this->translator->trans('changePwd.error',array(),'admin')]);
        }
    }

    /**
     * Check if pwd-file exists, directly redirect to login in not
     * 
     * @return none
     */
    private function _checkPwdFile()
    {
        // get the data-path
        if ($this->utils->checkManagementRunning(false)) {
            $hgPath = $this->hg->managementGetConfigurationEntry('main.conf','writeableDataPath');
            if (!$hgPath || $hgPath == '') {
                $hgPath = $this->hg->managementGetConfigurationEntry('main.conf','dataPath');    
            }
        } else {
            // homegear-management is not running or busy right now, so let's do it the classic way
            $hgPath = $this->utils->getDataPath();
        }
        $fs = new Filesystem();
        if (!$fs->exists($hgPath.'/defaultPassword.txt')) {
            $response = new RedirectResponse('/login');
            $response->send();
        }
    }
}