<?php
Namespace App\Controller\Admin\Inventory;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Validator\Validation;
use Symfony\Component\Validator\Constraints as Assert;
use App\Model\Admin\HomegearGateway;
use App\Model\Admin\HomegearDevice;
use App\Model\Admin\HomegearRoom;
use App\Utils\Utils;

class GatewayController extends Controller
{
    public function indexAction(Request $request, HomegearGateway $gateway)
    {
        return $this->render('Admin/Inventory/Gateway/list.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'gateways' => $gateway->list()
        ));
    }

    public function addAction(Request $request, HomegearGateway $gateway)
    {
        return $this->render('Admin/Inventory/Gateway/add.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'gateways' => $gateway->listFamilies()
        ));
    }

    public function getTypeAction($familyId, Request $request, HomegearGateway $gateway)
    {
        $types = $gateway->listTypes($familyId);
        return new JsonResponse($types);
    }

    public function searchAction($id, Request $request, HomegearGateway $gateway)
    {
        $result = $gateway->search($id);
        return new JsonResponse($result);
    }

    public function ssdpSearchAction($id, Request $request, HomegearGateway $gateway)
    {
        $result = $gateway->ssdpSearch($id);
        return new JsonResponse($result);
    }

    public function createAction($id, Request $request, HomegearGateway $gateway)
    {
        $params = $request->request->all();
        $result = $gateway->create($id,$params);
        if ($result != null) {
            // we have an error!
            return new JsonResponse($result);
        }
        return $this->redirectToRoute('admin.inventory.gateways',array(),301);
    }

    public function editAction($id, Request $request, HomegearGateway $gateway, Utils $utils)
    {
        if ($utils->checkFamilyExists($id)) {
            $expert = false;
            $params = $request->query->all();
            $userSettings = \Homegear\Homegear::getUserMetadata($this->getUser()->getUsername());
            if (array_key_exists('expert',$userSettings) && $userSettings['expert'] == 'true') {
                $expert = true;
            }
            return $this->render('Admin/Inventory/Gateway/edit.html.twig', array(
                'sect' => $request->attributes->get('sect'),
                'settings' => $gateway->getSettings($id,$params['gatewayId'],$expert),
                'mgmt' => $utils->checkManagementRunning(false),
                'expert' => $expert
            ));
        } else {
            return $this->redirectToRoute('admin.inventory.gateways',array(),301);
        }
    }

    public function updateAction($id, Request $request, HomegearGateway $gateway, Utils $utils)
    {
        $expert = false;
        $userSettings = \Homegear\Homegear::getUserMetadata($this->getUser()->getUsername());
        if (array_key_exists('expert',$userSettings) && $userSettings['expert'] == 'true') {
            $expert = true;
            //return $this->redirectToRoute('admin.inventory.gateways',array(),301);
        }
        $result = 0;
        $params = $request->query->all();
        $settings = $request->request->all();
        if (is_array($params) && array_key_exists('gatewayId', $params) && is_array($settings)) {
            // check if user tries to update IP-Address ... he has to be expert
            if (array_key_exists('host',$settings) && $expert === false) {
                unset($settings['host']);
            }
            $result = $gateway->updateSettings($id,$params['gatewayId'],$settings);
        }
        return new JsonResponse(array('success' => $result));
    }

    public function deleteAction($id, Request $request, HomegearGateway $gateway, HomegearDevice $devices)
    {
        $params = $request->query->all();
        $posted = $request->request->all();
        $gateways = $gateway->list();
        if (is_array($posted) && array_key_exists('deleteDevices',$posted) && $posted['deleteDevices'] == 'true') {
            // User want's to delete all devices from this gateway, too!
            $devices->deleteFromGateway($id,$params['gatewayId'],$gateways);
        }
        $result = $gateway->delete($id,$params['gatewayId']);
        return new JsonResponse(array('success' => 1));
    }

    public function getFormAction($familyId, $typeId, Request $request, HomegearGateway $gateway, Utils $utils)
    {
        $addMethods = $gateway->getSettingParameters($familyId, $typeId);
        return $this->render('Admin/Inventory/Gateway/gateway_form.html.twig', array(
            'familyId' => $familyId,
            'typeId' => $typeId,
            'addMethods' => $addMethods,
            'mgmt' =>$utils->checkManagementRunning(false)
        ));
    }

    public function searchRoomsAction($id, $ip, Request $request, HomegearGateway $gateway, HomegearRoom $room)
    {
        $params = $request->request->all();
        $result = $gateway->searchRooms($id, $ip);
        if (is_array($result) && array_key_exists('status',$result) && $result['status'] == 'success') {
            if (is_array($result['rooms']) && count($result['rooms']) > 0) {
                $currentRooms = $room->loadAll();
                if (is_array($currentRooms) && count($currentRooms) > 0) {
                    foreach ($currentRooms as $key => $value) {
                        foreach ($value['TRANSLATIONS'] as $lng => $name) {
                            $found = array_search(strtolower($name), array_column($result['rooms'], 'NameLower'));
                            if ($found !== False && array_key_exists($found,$result['rooms']) ) {
                                unset($result['rooms'][$found]);
                                $result['rooms'] = array_values($result['rooms']);
                            }
                        }
                    }
                    // reindex
                    foreach ($result['rooms'] as $key => $value) {
                        $room->create($value['Name'],$params['lang']);
                    }
                } else {
                    foreach ($result['rooms'] as $key => $value) {
                        $room->create($value['Name'],$params['lang']);
                    }
                }
            }
            $result = ['status' => count($result['rooms'])];
        }
        return new JsonResponse($result);
    }

    public function searchDeviceAction($id, Request $request, HomegearGateway $gateway)
    {
        $params = $request->query->all();
        $result = $gateway->searchDevices($id,$params);
        return new JsonResponse($result);
    }

    public function deviceInfoFileUploadAction($familyId, Request $request, HomegearGateway $gateway)
    {
        $id = $request->request->get('id');
        $file = $request->files->get('infofile');
        if ($file !== null) {
            $gateway->uploaddeviceInfoFile($familyId,$file);
        }
        return $this->redirectToRoute('admin.inventory.gateways.edit',array('id' => $familyId, 'gatewayId' => $id),301);
    }

    public function enableKNXForwarderAction($familyId, Request $request, HomegearGateway $gateway)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'fields' => array(
                'id' => new Assert\NotBlank,
                'knxForwarderIP' => new Assert\NotBlank,
                'knxForwarderPort' => new Assert\NotBlank
            ),
            'allowExtraFields' => true
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $gateway->enableKNXForwarder($familyId, $params['id'], $params['knxForwarderIP'],$params['knxForwarderPort']);
        }
        return $this->redirectToRoute('admin.inventory.gateways.edit',array('id' => $familyId, 'gatewayId' => $params['id']),301);
    }

    public function disableKNXForwarderAction($familyId, Request $request, HomegearGateway $gateway)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'fields' => array(
                'id' => new Assert\NotBlank
            ),
            'allowExtraFields' => true
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $gateway->disableKNXForwarder($familyId, $params['id']);
        }
        return $this->redirectToRoute('admin.inventory.gateways.edit',array('id' => $familyId, 'gatewayId' => $params['id']),301);
    }

}