<?php
Namespace App\Controller\Admin\Inventory;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Validator\Validation;
use Symfony\Component\Validator\Constraints as Assert;
use App\Model\Admin\HomegearRole;
use App\Model\Admin\HomegearDevice;

class RoleController extends Controller
{
    public function indexAction(Request $request, HomegearRole $roles)
    {
        $userSettings = \Homegear\Homegear::getUserMetadata($this->getUser()->getUsername());
        return $this->render('Admin/Inventory/Roles/list.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'roles' => $roles->loadAll(),
            'expert' => ((array_key_exists('expert',$userSettings) && $userSettings['expert'] == 'true') ? true : false)
        ));
    }

    public function editAction($id, Request $request, HomegearRole $role, HomegearDevice $device)
    {
        $userSettings = \Homegear\Homegear::getUserMetadata($this->getUser()->getUsername());
        $currentRole = $role->loadById($id);
        // get all known devices
        $devices = $device->loadAll();
        if (array_key_exists('devices',$currentRole) && $currentRole['devices'] !== NULL) {
            foreach($currentRole['devices'] as $k => $v) {
                // find device-data for devices in current room
                if ($v['ID'] == 'systemVariable') { continue; }
                $key = array_search(intval($v['ID']), array_column($devices, 'ID'));
                if (isset($key) && intval($key) !== NULL) {
                    $currentRole['devices'][$k]['name'] = $devices[$key]['name'];
                    if ($v['channel'] == -1) {
                        // only mark device as 'inGroup' if the whole device was added (channel == -1)
                        // so you are still able to add a complete device even if a single channel of this device was added before
                        $devices[$key]['inGroup'] = true;
                    }
                }
            }
        }
        return $this->render('Admin/Inventory/Roles/edit.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'role' => $currentRole,
            'allDevices' => $devices,
            'expert' => ((array_key_exists('expert',$userSettings) && $userSettings['expert'] == 'true') ? true : false)
        ));
    }

    public function updateAction($id, Request $request, HomegearRole $role)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'name' => new Assert\NotBlank,
            'goback' => new Assert\NotBlank
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $role->update($id,$params['name']);
        }
        // TODO: check result and redirect with flash
        if (array_key_exists('goback',$params) && $params['goback'] != '0') {
            return $this->redirectToRoute($params['goback']);
        } else {
            return $this->redirectToRoute('admin.inventory.roles.edit', array('id' => $id));
        }
    }

    public function deleteAction($id, Request $request, HomegearRole $role)
    {
        #$role->delete($id);
        return $this->redirectToRoute('admin.inventory.roles');
    }

    public function getDeviceToAdd(Request $request, HomegearDevice $device)
    {
        $params = $request->request->all();
        if (is_array($params) && array_key_exists('id',$params)) {
            $result = $device->load(intval($params['id']));
            mb_convert_variables('utf-8', 'ISO-8859-1', $result);
            return new JsonResponse($result);
        }
        return new JsonResponse(array('error' => 1));
    }

    public function addDeviceAction($id, Request $request, HomegearRole $role)
    {
        $params = $request->request->all();
        $role->addDevice($id,$params['device'],$params['channel'],$params['datapoint']);
        return new JsonResponse(array('success' => 1));
    }

    public function removeDeviceAction($id, Request $request, HomegearRole $role)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'deviceid' => new Assert\NotBlank,
            'channel' => new Assert\NotNull,
            'variable' => new Assert\NotNull,
        ));
        $params = $request->query->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $role->removeDevice($id,$params['deviceid'],$params['channel'],$params['variable']);
        }
        // TODO: check result and redirect with flash
        return $this->redirectToRoute('admin.inventory.roles.edit', array('id' => $id));
    }
}