<?php
Namespace App\Controller\Admin\Settings;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use App\Model\Admin\HomegearSystem;
use App\Model\Admin\HomegearModule;
use App\Utils\Utils;

class SystemController extends Controller
{

    public function indexAction(Request $request, HomegearSystem $system, HomegearModule $module, Utils $utils)
    {
        $userSettings = \Homegear\Homegear::getUserMetadata($this->getUser()->getUsername());
        if (array_key_exists('expert',$userSettings) && $userSettings['expert'] == 'true') {
            // get settings for expert users
            $influxdbSettings = $system->getInfluxdbSettings();
            $mqttSettings = $system->getMqttSettings();
            $reset = 1;
        }
        // check for sonos-module for tts settings
        $modules = $module->listInstalled();
        $key = array_search(6, array_column($modules, 'FAMILY_ID'));
        if ($key !== false && $modules[$key]['LOADED'] !== false) {
            $ttsSettings = $system->getTtsSettings();
        }
        return $this->render('Admin/Settings/System/index.html.twig',array(
            'sect' => $request->attributes->get('sect'),
            'user' => $userSettings,
            'networkSettings' => $system->getNetworkSettings(),
            'networkActiveConfig' => $utils->getIpAddresses(),
            'influxdbSettings' => (isset($influxdbSettings) ? $influxdbSettings : null),
            'mqttSettings' => (isset($mqttSettings) ? $mqttSettings : null),
            'ttsSettings' => (isset($ttsSettings) ? $ttsSettings : null),
            'reset' => (isset($reset) ? $reset : null)
        ));
    }

    public function updateNetworkAction(Request $request, HomegearSystem $system)
    {
        $params = $request->request->all();
        $result = $system->setNetworkSettings($params);
        return new JsonResponse($result);
    }

    public function updateInfluxDBAction(Request $request, HomegearSystem $system)
    {
        $params = $request->request->all();
        $result = $system->setInfluxdbSettings($params);
        if ($result['result'] == 'success') {
            $result = $system->restartService('homegear-influxdb');
            return new JsonResponse($result);
        }
        return new JsonResponse($result);
    }

    public function updateMqttAction(Request $request, HomegearSystem $system)
    {
        $params = $request->request->all();
        $result = $system->setMqttSettings($params);
        if ($result['result'] == 'success') {
            return new JsonResponse($result);
        }
        return new JsonResponse($result);
    }

    public function updateTtsAction(Request $request, HomegearSystem $system, HomegearModule $module)
    {
        $params = $request->request->all();
        $result = $system->setTtsSettings($params);
        if ($result['result'] == 'success') {
            $module->reload('mod_sonos.so');
            return new JsonResponse($result);
        }
        return new JsonResponse($result);
    }

    public function resetAction(HomegearSystem $system)
    {
        $system->doReset();
        return new JsonResponse(['status' =>'success']);
    }
}