<?php
Namespace App\Model\Admin;

use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;
use App\Model\HomegearBuilding as BaseBuilding;

/**
 * Provides admin functions for Homegear room management
 *
 */
class HomegearBuilding extends BaseBuilding
{
    public function __construct(ParameterBagInterface $params)
    {
        parent::__construct($params);
    }

    /**
     * Create a new building
     * 
     * @return int
     */
    public function create($name, $lang = null)
    {
        $data = array();
        if ($lang === null) { 
            $data = [$this->defaultLanguage => $name];
        } else {
            $data = [$this->defaultLanguage => $name, $lang => $name];
        }
        try {
            $newId = $this->hg->createBuilding($data);
        }
        catch(\Homegear\HomegearException $e) {
            return $e;
        }
        return $newId;
    }

    /**
     * Update a building name
     * 
     * @return none
     */
    public function update($id = null, $name = null)
    {
        if ($id === null || $name === null) { return; }
        if (!is_array($name)) {
            $name = [$this->defaultLanguage => "$name"];
        }
        foreach ($name as $l => $n) {
            if ($name == '') {
                unset($name[$l]);
            }
        }
        // Update name
        $this->hg->updateBuilding(intval($id),$name);
    }

    /**
     * Delete a building
     * 
     * @return none
     */
    public function delete($id) {
        // delete all stories in building
        $storiesInBuilding = $this->hg->getStoriesInBuilding(intval($id));
        if (count($storiesInBuilding) > 0) {
            foreach($storiesInBuilding as $k => $v) {
                $this->hg->removeStoryFromBuilding(intval($id),intval($v));
            }
        }
        $this->hg->deleteData(parent::componentBuildingGridSort,$id);
        $this->hg->deleteBuilding(intval($id));
    }

    /**
     * Update sorting of buildings
     * 
     * @return none
     */
    public function updateSort($id = null, $position = null) {
        if ($id === null || $position === null) { return; }
        $metadata = $this->hg->getBuildingMetadata(intval($id));
        if(!is_array($metadata)) $metadata = array();
        $metadata['position'] = intval($position);
        $this->hg->setBuildingMetadata(intval($id), $metadata);
    }

    /**
     * Adds stories to a building
     * 
     * @return none
     */
    public function addStory($id = null, $stories = null)
    {
        if ($id === null || $stories === null)  { return; }
        if (is_array($stories) && (count($stories) > 0)) {
            foreach ($stories as $story) {
                $this->hg->addStoryToBuilding(intval($id),intval($story));
            }
        }
    }

    /**
     * Remove a story from a building
     * 
     * @return none
     */
    public function removeStory($id = null, $storyId = null) 
    {
        if ($id === null || $storyId === null)  { return; }
        // get sorting of rooms in current story
        $sort = json_decode($this->hg->getData(parent::componentBuildingGridSort,"$id"),true);
        // TODO: add this check to all functions using sort-table!
        if ($sort !== null && is_array($sort)) {
            // check if target-device is in sorting-array
            $key = array_search("$storyId",$sort);
            if ($key !== false) {
                // room found, delete it from array and save again
                unset($sort[$key]);
                $this->hg->setData(parent::componentBuildingGridSort,$id,json_encode($sort));
            }
        }
        $this->hg->removeStoryFromBuilding(intval($id),intval($storyId));
    }

    public function updateStorySort($id = null, $story = null, $position = null) {
        if ($id === null || $story === null || $position === null) { return; }
        // get current sort-data
        $sort = json_decode($this->hg->getData(parent::componentBuildingGridSort,"$id"),true);
        // check if target-room is in array already
        if (is_array($sort)) {
            $key = array_search("$story",$sort);
            if ($key !== false) {
                // target-device found, delete it from array
                unset($sort[$key]);
            }
        }
        // set new position
        $sort[$position] = "$story";
        // save new sort-data
        $this->hg->setData(parent::componentBuildingGridSort,$id,json_encode($sort));
    }
}