<?php
Namespace App\Model\Admin;

use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;
use App\Model\HomegearProfile as BaseProfile;

/**
 * Provides admin functions for Homegear profile management
 *
 */
class HomegearProfile extends BaseProfile
{
    public function __construct(ParameterBagInterface $params)
    {
        parent::__construct($params);
    }

    public function create($name = null, $lang = null)
    {
        if ($name === null) { return; }
        if ($lang === null) {
            $profileName = [$this->defaultLanguage => $name];
        } else {
            $profileName = [$this->defaultLanguage => $name, $lang => $name];
        }
        return $this->hg->addVariableProfile($profileName,['values' => [], 'roles' => []]);
    }

    public function update($id = null, $params = null)
    {
        if ($id === null || $params === null || !array_key_exists('name', $params)) { return; }
        foreach ($params['name'] as $locale => $name) {
            $profileName[$locale] = $name;
        }
        $currentProfile = $this->loadById($id);
        if (array_key_exists('room', $params) && $params['room'] != '') {
            $this->hg->updateVariableProfile(intval($id),$profileName,['room' => intval($params['room']), 'values' => (array_key_exists('values', $currentProfile) ? $currentProfile['values'] : []), 'roles' => (array_key_exists('roles',$currentProfile) ? $currentProfile['roles'] : [])]);
        } else {
            $this->hg->updateVariableProfile(intval($id),$profileName,['values' => (array_key_exists('values', $currentProfile) ? $currentProfile['values'] : []), 'roles' => (array_key_exists('roles',$currentProfile) ? $currentProfile['roles'] : [])]);
        }
    }

    public function delete($id = null)
    {
        $this->hg->deleteVariableProfile(intval($id));
    }

    public function loadById($id = null)
    {
        if ($id === null) { return; }
        return $this->hg->getVariableProfile(intval($id));
    }

    public function addItem($id = null, $params = null)
    {
        if ($id === null || !is_array($params)) { return; }
        if ($params['item'] == 'variable') {
            if ($params['id'] == 0 && $params['channel'] == '-1') {
                $type = strtoupper($params['device']['type']);
            } else {
                $type = $params['device']['CHANNELS'][$params['channel']]['PARAMSET'][$params['variable']]['TYPE'];
            }
        }
        if ($params['item'] == 'role') {
            $type = strtoupper($params['type']);
        }
        $params['value'] = $this->_formatType($params['value'], $type);
        $currentProfile = $this->loadById($id);
        if ($params['item'] == 'variable') {
            $currentProfile['values'][] = ['peerId' => intval($params['id']), 'channel' => intval($params['channel']), 'variable' => $params['variable'], 'value' => $params['value']];
        }
        if ($params['item'] == 'role') {
            $currentProfile['roles'][] = ['role' => intval($params['id']), 'value' => $params['value']];
        }
        if (array_key_exists('room', $currentProfile) && $currentProfile['room'] != '') {
            $this->hg->updateVariableProfile(intval($id),[],['room' => $currentProfile['room'], 'values' => (array_key_exists('values', $currentProfile) ? $currentProfile['values'] : []), 'roles' => (array_key_exists('roles',$currentProfile) ? $currentProfile['roles'] : [])]);
        } else {
            $this->hg->updateVariableProfile(intval($id),[],['values' => (array_key_exists('values', $currentProfile) ? $currentProfile['values'] : []), 'roles' => (array_key_exists('roles',$currentProfile) ? $currentProfile['roles'] : [])]);
        }
    }

    public function updateItem($id = null, $params = null)
    {
        if ($id === null || !is_array($params)) { return; }
        if ($params['item'] == 'variable') {
            if ($params['id'] == 0 && $params['channel'] == '-1') {
                $type = strtoupper($params['device']['type']);
            } else {
                $type = $params['device']['CHANNELS'][$params['channel']]['PARAMSET'][$params['variable']]['TYPE'];
            }
        }
        if ($params['item'] == 'role') {
            $type = strtoupper($params['type']);
        }
        $params['value'] = $this->_formatType($params['value'], $type);
        $currentProfile = $this->loadById($id);
        if ($params['item'] == 'variable') {
            if (!array_key_exists('id', $params) || !array_key_exists('channel', $params) || !array_key_exists('variable', $params)) { return; }
            foreach ($currentProfile['values'] as $key => $value) {
                if ($value['peerId'] == $params['id'] && $value['channel'] == $params['channel'] && $value['variable'] == $params['variable']) {
                    $currentProfile['values'][$key] = ['peerId' => intval($params['id']), 'channel' => intval($params['channel']), 'variable' => $params['variable'], 'value' => $params['value'], 'wait' => $this->_formatType($params['wait'],'BOOL'), 'ignoreValueFromDevice' => $this->_formatType($params['ignoreValueFromDevice'],'BOOL'), 'deviceRefractoryPeriod' => intval($params['deviceRefractoryPeriod'])];
                }
            }
        }
        if ($params['item'] == 'role') {
            if (!array_key_exists('id', $params)) { return; }
            foreach ($currentProfile['roles'] as $key => $value) {
                if ($value['role'] == $params['id']) {
                    $currentProfile['roles'][$key] = ['role' => intval($params['id']), 'value' => $params['value'], 'wait' => $this->_formatType($params['wait'],'BOOL'), 'ignoreValueFromDevice' => $this->_formatType($params['ignoreValueFromDevice'],'BOOL'), 'deviceRefractoryPeriod' => intval($params['deviceRefractoryPeriod'])];
                }
            }
        }
        if (array_key_exists('room', $currentProfile) && $currentProfile['room'] != '') {
            $this->hg->updateVariableProfile(intval($id),[],['room' => $currentProfile['room'], 'values' => (array_key_exists('values', $currentProfile) ? $currentProfile['values'] : []), 'roles' => (array_key_exists('roles',$currentProfile) ? $currentProfile['roles'] : [])]);
        } else {
            $this->hg->updateVariableProfile(intval($id),[],['values' => (array_key_exists('values', $currentProfile) ? $currentProfile['values'] : []), 'roles' => (array_key_exists('roles',$currentProfile) ? $currentProfile['roles'] : [])]);
        }
    }

    public function removeItem($id = null, $params = null)
    {
        if ($id === null || $params === null || !is_array($params) || !array_key_exists('item', $params)) { return; }
        $currentProfile = $this->loadById($id);
        if ($params['item'] == 'variable') {
            if (!array_key_exists('peerId', $params) || !array_key_exists('channel', $params) || !array_key_exists('variable', $params)) { return; }
            foreach ($currentProfile['values'] as $key => $value) {
                foreach ($params['peerId'] as $k => $peer) {
                    if ($value['peerId'] == $peer && $value['channel'] == $params['channel'][$k] && $value['variable'] == $params['variable'][$k]) {
                        unset($currentProfile['values'][$key]);
                    }
                }
            }
            $currentProfile['values'] = array_values($currentProfile['values']);
        }
        if ($params['item'] == 'role') {
            if (!array_key_exists('roleId', $params)) { return; }
            foreach ($currentProfile['roles'] as $key => $value) {
                foreach ($params['roleId'] as $k => $role) {
                    if ($value['role'] == $role) {
                        unset($currentProfile['roles'][$key]);
                    }
                }
            }
            $currentProfile['roles'] = array_values($currentProfile['roles']);
        }
        if (array_key_exists('room', $currentProfile) && $currentProfile['room'] != '') {
            $this->hg->updateVariableProfile(intval($id),[],['room' => $currentProfile['room'], 'values' => (array_key_exists('values', $currentProfile) ? $currentProfile['values'] : []), 'roles' => (array_key_exists('roles',$currentProfile) ? $currentProfile['roles'] : [])]);
        } else {
            $this->hg->updateVariableProfile(intval($id),[],['values' => (array_key_exists('values', $currentProfile) ? $currentProfile['values'] : []), 'roles' => (array_key_exists('roles',$currentProfile) ? $currentProfile['roles'] : [])]);
        }
    }

    private function _formatType($value = null, $type = null)
    {
        if ($value === null || $type === null) { return; }
        if ($type == 'BOOL') {
            if ($value == "false") { $value = 0; }
            return boolval($value);
        } elseif ($type == 'ENUM' || $type == 'INTEGER') {
            return intval($value);
        } elseif ($type == 'FLOAT') {
            return floatval($value);
        } elseif ($type == 'STRING') {
            return strval($value);
        } elseif ($type == 'ACTION') {
            if ($value == "false") { $value = 0; }
            return boolval($value);
        }
    }
}