<?php
Namespace App\Model\Admin;

use App\Model\HomegearVariable as BaseVariable;

/**
 * Provides admin functions for Homegear variable management
 *
 */
class HomegearVariable extends BaseVariable
{
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Create a system variable
     * 
     * @return int
     */
    public function create($name = null, $type = null, $flag = null, $value = null) {
        if ($name === null || $type === null) { return; }
        if ($flag === null) { $flag = 0; } // visible, readwrite
        switch ($type) {
            case "string":
                if ($value == null) { $value = ''; }
                $result = $this->hg->setSystemVariable($name,"$value", intval($flag));
            break;
            case "int":
                if ($value == null) { $value = 1; }
                $result = $this->hg->setSystemVariable($name,intval($value), intval($flag));
            break;
            case "float":
                if ($value == null) { $value = 0.12; }
                $result = $this->hg->setSystemVariable($name,floatval($value), intval($flag));
            break;
            case "bool":
                if ($value == null) { $value = 1; }
                $result = $this->hg->setSystemVariable($name, filter_var($value, FILTER_VALIDATE_BOOLEAN), intval($flag));
            break;
            case "array":
                if ($value == null) { $value = []; }
                $result = $this->hg->setSystemVariable($name, $value, intval($flag));
            break;
            case "struct":
                if ($value == null) { $value = [[]]; }
                $result = $this->hg->setSystemVariable($name, $value, intval($flag));
            break;
        }
        /*if ($result == -32700) {
            return $this->outputMessage("Data could not be encoded to binary format.",0);
        } elseif ($result == -32602) {
            return $this->outputMessage("Name is longer than 250 characters.",0);
        } elseif ($result == -32500) {
            return $this->outputMessage("The limit of 1 million metadata entries has been reached.",0);
        }*/
        return $result;
    }

    /**
     * Load a specific system variable
     * 
     * @return array()
     */
    public function loadByName($name = null) {
        if ($name === null) { return; }
        $v = $this->hg->getSystemVariable($name);
        $flag = $this->hg->getSystemVariableFlags($name);
        $type = 'struct';
        $value = print_r($v, true);
        if (is_string($v)) { $type = 'string'; $value = $v; }
        if (is_int($v)) { $type = 'int'; $value = intval($v); }
        if (is_float($v)) { $type = 'float'; $value = floatval($v); }
        if (is_bool($v)) { $type = 'bool'; $value = boolval($v); }
        if (is_array($v)) { $type = 'array'; $value = print_r($v, true); }
        return ['name' => $name, 'url' => base64_encode($name), 'type' => $type, 'flag' => ($flag ? $flag : 0), 'value' => $value];
    }

    /**
     * Delete a system variable
     * 
     * @return none
     */
    public function delete($name = null) {
        if ($name === null) { return; }
        $this->hg->deleteSystemVariable($name);
    }

    public function getFromCcu($params = null)
    {
        if ($params === null || !is_array($params)) { return false; }
        $script = "object oSysvar; string sSysvarId; boolean bFirst = true; boolean bFirstSecond = true; Write(\"{\"); Write('\"Systemvariables\":['); foreach (sSysvarId, dom.GetObject(ID_SYSTEM_VARIABLES).EnumUsedIDs()) { if (bFirst == false) { WriteLine(','); } else { bFirst = false; } oSysvar = dom.GetObject(sSysvarId); Write(\"{\"); Write('\"ID\":\"' # oSysvar.ID()); Write('\",\"Description\":\"' # oSysvar.DPInfo()); Write('\",\"Name\":\"' # oSysvar.Name()); if (oSysvar.ValueType() == 20) { var value = oSysvar.Value(); var valueLength = value.Length(); var currentIndex = 0; var correctedValue = \"\"; while(currentIndex < valueLength) { var char1 = value.Substr(currentIndex, 1); if(char1 == \"\\\"\") { correctedValue = correctedValue # \"'\"; } else { if(char1 == \"\n\") { correctedValue = correctedValue # \" \"; } else { correctedValue = correctedValue # char1; } } currentIndex = currentIndex +1; } Write('\",\"VariableI\":\"' # correctedValue); Write('\", \"Value\": \"' # correctedValue); } else { if(oSysvar.ValueSubType() == 6) { Write('\",\"VariableI\":\"' # oSysvar.AlType()); Write('\", \"Value\": \"' # oSysvar.Value()); } else { Write('\",\"VariableI\":\"' # oSysvar.Value()); Write('\", \"Value\": \"' # oSysvar.Value()); } } Write('\", \"ValueType\": \"' # oSysvar.ValueType()); if (oSysvar.ValueType() == 16) { Write('\", \"ValueList\": \"' # oSysvar.ValueList()); } else { Write('\", \"ValueList\": \"'); } if (oSysvar.ValueType() == 4) { Write('\", \"Min\": \"' # oSysvar.ValueMin()); Write('\", \"Max\": \"' # oSysvar.ValueMax()); } else { Write('\", \"Min\": \"'); Write('\", \"Max\": \"'); } Write('\", \"Unit\": \"' # oSysvar.ValueUnit()); Write('\", \"Type\": \"' # oSysvar.ValueType()); Write('\", \"Subtype\": \"' # oSysvar.ValueSubType()); Write('\", \"Timestamp\": \"' # oSysvar.Timestamp()); if (oSysvar.ValueType() == 2) { Write('\", \"ValueName1\": \"' # oSysvar.ValueName0()); Write('\", \"ValueName2\": \"' # oSysvar.ValueName1()); } else { Write('\", \"ValueName1\": \"'); Write('\", \"ValueName2\": \"'); } Write('\", \"Visible\": \"' # oSysvar.Visible() # '\"'); Write(\"}\"); } Write(']}');";
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10); // connect timeout
        curl_setopt($ch, CURLOPT_TIMEOUT, 300); // connect timeout
        curl_setopt($ch, CURLOPT_URL, $params['ip'].':8181/tclrega.exe');
        curl_setopt($ch, CURLOPT_HEADER, 0);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array('Expect:'));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS,$script);
        $result = curl_exec($ch);
        $result = utf8_encode($result);
        $error_string = curl_error($ch);
        $error = curl_errno($ch);
        curl_close($ch);
        $error = explode("\n",$error);
        if ($error[0] == 0) {
            $result = trim(preg_replace('/\<xml\>(.+)\<\/xml\>/', '', $result));
            return json_decode($result);
        }
        return false;
    }
}